import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)
import java.awt.Color;
import java.util.*;
/**
 * Write a description of class Desk here.
 * 
 * @author John Littlewood
 * @version preA
 */
public class Desk  extends World
{
    private static final int BOTTOM = 380;
    private static final int TOP = 50;
    private static final String[] possLetters = {"a","b","c","d","e","f","g","h","i","j","k","l","m","n","o","p","q","r","s","t","u","v","w","x","y","z"};
    private static final String[] specials = {"@","'","<",",",".",">","?","/","=","+","#","~","-","_",";",":"};
    
    private String keyPress;
    private int level;
    private int newLetterDelay;
    private int newLetterCount;
        
    private LevelCounter levelCounter;
    private NumericalCounter scoreCounter;
    private LinearCounter lifeCounter;
    
    private ArrayList<Letter> exploders;
    
    private HashMap<String, Integer> frequencies;
    
    private boolean gameOver = false;
    /**
     * Constructor for objects of class Desk.
     * 
     */
    public Desk()
    {    
        super(500, 430, 1);
        GreenfootImage img = getBackground();
        
        for (int i=0; i<((BOTTOM-TOP)/2); i++) {
            img.setColor(new Color(0,0,(((BOTTOM-TOP)/2)-i)/2));
            img.fillRect(0,TOP+3+i*2,getWidth(),2);
        }
        
        img.setColor(new Color(255,0,0));
        img.fillRect(0, TOP, getWidth(), 3);
        img.fillRect(0, BOTTOM, getWidth(), 3);
        level = 1;
        
        levelCounter = new LevelCounter(5);
        addObject(levelCounter, 250,30);        
        newLetterDelay = 500 - (level) * 100;
        newLetterCount = newLetterDelay -50;
        
        scoreCounter = new NumericalCounter(0);
        addObject(scoreCounter, 450,410);
        
        GreenfootImage lifeCounterImg = new GreenfootImage("herz.png");
        lifeCounterImg.scale(25,25);
        lifeCounter = new LinearCounter(lifeCounterImg, new GreenfootImage(1,1), 6,3);
        addObject(lifeCounter, 100, 410);
                
        exploders = new ArrayList<Letter>();
        
        frequencies = new HashMap<String, Integer>();
        
        for (String letter : possLetters) {
            frequencies.put(letter, 0);
        }
        for (String cha : specials) {
            frequencies.put(cha, 0);
        }
    }
    
    public int getBottom()
    {
        return BOTTOM;
    }
    
    public int getLevel()
    {
        return level;
    }
    
    public void addLevel()
    {
        level++;
        newLetterDelay = calculateLetterDelay();
    }
    
    public void act()
    {
        if (!gameOver) {
            String keyPress = Greenfoot.getKey();
            List<Letter> letters = getObjects(Letter.class);
            boolean gotHit = false;
            int i=0;
            
            while (i<letters.size() && gotHit == false) {
                if (letters.get(i).getLetter().equals(keyPress)) {
                    letters.get(i).hit();
                    gotHit = true;
                    
                    frequencies.put(keyPress, frequencies.get(keyPress) +1);
                }
                i++;
            }
            
            if (gotHit == false && keyPress != null) { 
                Greenfoot.playSound("wrongletter.wav");
                addALetter();
            }
            
            newLetterCheck();
        }    
        exploders();
    }
    
    public String lastKey()
    {
        return keyPress;
    }
    
    private void newLetterCheck()
    {
        if (newLetterCount >= newLetterDelay) {
            addALetter();
            newLetterCount = 0;
        }
        else {
            newLetterCount++;
        }
    }
    
    private void addALetter()
    {
        Letter letter;
        
        if (Greenfoot.getRandomNumber(100) == 54) {
            letter = new SpecialCharacter(specials[Greenfoot.getRandomNumber(16)], level);
        }
        else {
            String randLetter = possLetters[Greenfoot.getRandomNumber(26)];
            letter = new Letter(randLetter, level);
        }
        
        int randY = Greenfoot.getRandomNumber(getWidth()-60)+30;
        addObject(letter, randY, TOP);
    }
    
    public void addTestLetters()
    {
        for (int i=0; i<26; i++) {
            addObject(new Letter(possLetters[i],1), Greenfoot.getRandomNumber(1000), Greenfoot.getRandomNumber(300)+TOP);
        }
    }
    
    public void addHit()
    {
        levelCounter.addHit();
        addScore(getLevel() * 100);
    }
    
    private int calculateLetterDelay()
    {
        if (newLetterDelay == 1) {
            return 1;
        }
        else if (newLetterDelay > 1 && newLetterDelay <= 50) {
            return newLetterDelay -1;
        }
        else if (newLetterDelay > 50 && newLetterDelay <= 100) {
            return newLetterDelay -5;
        }
        else if (newLetterDelay > 100 && newLetterDelay <= 150) {
            return newLetterDelay -20;
        }
        else if (newLetterDelay > 150 && newLetterDelay <= 300) {
            return newLetterDelay -50;
        }
        else {
            return 300;
        }
    }
    
    public void addScore(int value)
    {
        scoreCounter.add(value);
    }
    
    public int getScore()
    {
        return scoreCounter.getCount();
    }
    
    public void miss()
    {
        loseALife();
        
        explodeAllLetters();
    }
    
    private void loseALife()
    {
        if (!lifeCounter.subtractCount()) {
            gameOver();
        }
    }
    
    private void addALife()
    {
        lifeCounter.addCount();
    }
    
    private void exploders()
    {
        if (!exploders.isEmpty()) {
            Letter exploder = exploders.get(0);
            
            if (exploder.getWorld() != null) {
                exploders.get(0).explode(false);
            }    
            exploders.remove(0);
        }
    }
    
    private void gameOver()
    {
        gameOver = true;
        addObject(new GameOverScreen(frequencies, scoreCounter.getCount(), level), getWidth() / 2, getHeight() / 2);
    }
    
    public void bonusPrize()
    {
        if (!lifeCounter.addCount()) {
            explodeAllLetters();
        }
    }
    
    private void explodeAllLetters()
    {
        for (Object letter : getObjects(Letter.class) ) {
            exploders.add((Letter) letter);
        }
    }
            
            
}
