import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)
import java.awt.*;
/**
 * The letters to be typed by the player
 * 
 * @author John Littlewood
 */
public class Letter  extends Actor
{
    private final static int BASE_SCORE = 500; // the score for hitting a letter when the letter first appears
    
    private int scoreForHit; // the score for a hitting a letter at the current moment
    
    private String letter; // the letter represented in this instance
    
    private Color colour;
    
    private int fallDelay; // the turns to wait until falling again (controls speed)
    private int fallCount; // tracks number of turns since the last fall
    
    private int level;
    
    
    public Letter()
    {
        this("a",1);
    }
    
    public Letter(String letter, int level)
    {
        this.letter = letter;
        
        colour = new Color(Greenfoot.getRandomNumber(200) +55, Greenfoot.getRandomNumber(200) +55, Greenfoot.getRandomNumber(200) +55);
        
        GreenfootImage img = new GreenfootImage(50,50);
        img.clear();
        img.setColor(colour);
        img.setFont(new Font("sansserif", Font.BOLD, 32));
        img.drawString(letter, 0,40);
        setImage(img);
        

        fallDelay = 5 - level/10;
        fallCount = 0;
        this.level=level;
        
        scoreForHit = BASE_SCORE;
    }
    
    public void act() 
    {
        if (isAtBottom()) {
            Greenfoot.playSound("Explosion.wav");
            hitBottom();
        }
        else if (fallCount >= fallDelay) {
            moveDown();
            fallCount = 0;
        }
        
        fallCount++;
    }
    
    private boolean isAtBottom()
    {
        Desk desk = (Desk) getWorld();
        return desk.getBottom() <= getY();
    }
    
    private void moveDown()
    {
        setLocation(getX(), getY() +1);
    }
    
    private void hitBottom()
    {
        Desk desk = (Desk) getWorld();
        explode(false);  
        desk.miss();  
    }
    
    public void hit()
    {
        Desk desk = (Desk) getWorld();
        desk.addHit();
        explode(true);
    }
    
    public void explode(boolean playSound)
    {
        if (playSound) {
            Greenfoot.playSound("photongun.wav");
        }
        
        World world = getWorld();
        for (int i=0; i<50; i++) {
            world.addObject(new Debris(colour), getX(), getY());
        }
        world.removeObject(this);
    }
    
    public String getLetter()
    {
        return letter;
    }
}
