import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)
import java.awt.Color;
/**
 * A graphic for counting letters hit and the current level
 * 
 * @author John Littlewood
 */
public class LevelCounter  extends LinearCounter
{
    private int lvlUpHits; //number of hits before a level up
    
    private int level;
    
    public LevelCounter(int lvlUpHits)
    {
        super(lvlUpHits, 0, "1");
        this.lvlUpHits = lvlUpHits;
        level = 1;
    }
    
    /**
     * add a hit to the Counter
     */
    public void addHit()
    {
        if (!addCount()) {
            levelUp();
        }
    }
    
    private void levelUp()
    {
        level ++;

        levelUpAnimation();

        setCurrent(0);
        
        if (level % 5 == 0) {
            lvlUpHits += 5;
            setCap(lvlUpHits);
        }
        
        setLabel("" + level);
        
        Desk desk = (Desk) getWorld();
        desk.addLevel();
    }
    
    /**
     * Animation on the Counter when levelling up. causes each square to appear to explode
     */
    public void levelUpAnimation()
    {
        GreenfootImage img = getImage();
        
        Greenfoot.playSound("glass.wav");
        
        for (int i=0; i<lvlUpHits; i++) {
            Color colour = img.getColorAt(i*6,5);
            
            World world = getWorld();
            for (int j=0; j<10; j++) {
                world.addObject(new Debris(colour), getX()-img.getWidth()/2+i*12+5, getY()+5);
            }
        }
    }
    
    
}
