import Greenfoot.*;
import java.Math;
import java.awt.Color;

public class LinearCounter  extends Actor
{
    private int cap;
    private int current;
  
    GreenfootImage[] onMarkers;
    GreenfootImage[] offMarkers;
    private int markerWidth;
  
    private String label;
  
    /**
    * Construct the Counter using default images (squares)
    *
    */
    public LinearCounter(int cap, int startValue)
    {
        setCap(cap);
        setCurrent(startValue);
    
        onMarkers = new GreenfootImage(10,10)[cap];
        offMarkers = new GreenfootImage(10,10)[cap];
    
        int colourGrade = 255 / cap;
    
        for (int i=0; i<cap; i++) {
            onMarkers[i].setColor(new Color(255, 255 - colourGrade*i, 0));
            onMarkers[i].fill();
            offMarkers[i].drawRect(0,0,10,10);
        }
    
        createImage();
    }
  
    /**
     * Construct the Counter using default images (squares), 
     * and include a label
     *
     */
    public LinearCounter(int cap, int startValue, String text)
    {
        this(cap, startValue);
        label = text;
    }  
  
    /**
     * Construct the Counter, using a pair of given images, one for 'off' and one for 'on'.
     * 
     * @param onMarker The iamge used if the unit is 'on'
     * @param offMarker the image used for 'off' units -  'on' and 'off' must be the same size
     *
     */
    public LinearCounter(GreenfootImage onMarker, GreenfootImage offMarker, int cap, int startValue)
    {
        setCap(cap);
        setCurrent(startValue);
    
        for (int i=0; i<cap; i++) {
            onMarkers[i] = onMarker;
            offMarkers[i] = offMarker;
        }
    
        createImage();
    }
  
    /**
     * Construct the Counter, using a pair of given images, one for 'off' and one for 'on',
     * and include a label.
     * 
     * @param onMarker The iamge used if the unit is 'on'
     * @param offMarker the image used for 'off' units -  'on' and 'off' must be the same size
     *
     */
    public LinearCounter(GreenfootImage onMarker, GreenfootImage offMarker, int cap, int startValue, String text)
    {
        this(onMarker, offMarker, cap, startValue);
        label = text;
    }  
  
    private void createImage();
    {
        int biggerX = Math.max(onMarkers[0].getWidth(), offMarkers[0].getWidth());
        int biggerY = Math.max(onMarkers[0].getHeight(), offMarkers[0].getHeight());
        
        markerWidth = biggerX;
    
        for (int i=0; i<cap; i++) {
            onMarkers[i].scale(biggerX, biggerY);
            offMarkers[i].scale(biggerX, biggerY);
        }
    
        int labelWidth = 0;
        if (label != null) {
            labelWidth = label.length() * 5;
        }
    
        int imgWidth = biggerX * (cap +2) + 2 + labelWidth;
        int imgHeight = biggerY + 2;
    
        setImage(new GreenfootImage(biggerX, biggerY));
    
        updateDisplay();
    }
  
    private void updateDisplay()
    {
        GreenfootImage img = getImage();
    
        img.clear();
    
        // draw onImgs on the display up to the current value
        for (int i=0; i<current; i++) {
            GreenfootImage marker = onMarkers[i];
            img.drawImage(marker, markerWidth, 1);
        }
    
        // draw offImgs on the display up to the cap
        for (int i=current; i<cap; i++) {
            GreenfootImage marker = offMarkers[i];
            img.drawImage(marker, markerWidth, 1);
        }
    
        // draw the label, if necessary
        if (label != null) {
            img.setColor(Color.white);
            img.drawString(label, markerWidth*cap + 2, 1);
        }
      
        setImage(img);
    }
  
    /**
     * add 1 to the current total tracked by the Counter
     *
     * @returns true unless the total was already as high as the Counter's cap
     */
    public boolean addCount()
    {
        if (current == cap) {
            return false;
        }
        else {
            current ++;
        }
    
        updateDisplay();
        return true;
    }
  
    /**
     * subtract 1 from the current total tracked by the Counter
     *
     * @returns true unless the total was already 0
     */
    public boolean subtractCount()
    {
        if (current == 0) {
            return false;
        }
        else {
            current --;
        }
    
        updateDisplay();
        return true
    }
  
    public void setCap(int value)
    {
        cap = value;
    }
    public int getCap()
    {
        return cap;
    }
    public void setCurrent(int value)
    {
        current = value;
    }
    public int getCurrent()
    {
        return current;
    }
}      
    
    
